@@ -1,14 +1,10 @@
-from base64 import b64encode
-import calendar
-from datetime import datetime
 import logging
 import json
-from hashlib import sha512
-import hmac
 import re
 import requests
 
 from .account import Account
+from .auth import AmbisafeAuth
 from .containers import Container
 from .exc import ClientError, ServerError
 from .transactions import BaseTransaction, Wallet4Transaction, RecoveryTransaction, GrantTransaction
@@ -17,13 +13,22 @@ logger = logging.getLogger('ambisafe')
 
 
 class Client(object):
-    def __init__(self, ambisafe_server, secret, api_key, api_secret):
+    def __init__(self, ambisafe_server, secret, api_key, api_secret, account_id_prefix=''):
         if not re.match(r'^(http|https)://', ambisafe_server):
             raise ValueError('ambisafe_server should be http/https URI')
         self.ambisafe_server = ambisafe_server
         self.secret = secret
         self.api_key = api_key
         self.api_secret = api_secret
+        self.account_id_prefix = account_id_prefix + ':' if account_id_prefix else ''
+
+    def get_prefixed_account_id(self, account_id):
+        return '{}{}'.format(self.account_id_prefix, account_id)
+
+    def remove_prefix_from_account_id(self, account_id):
+        if self.account_id_prefix:
+            return account_id[len(self.account_id_prefix):]
+        return account_id
 
     def create_simple_account(self, account_id, currency='BTC'):
         """
@@ -36,7 +41,7 @@ class Client(object):
                      .format(account_id, currency))
 
         response = self.make_request('POST', '/accounts', json.dumps({
-            'id': str(account_id),
+            'id': self.get_prefixed_account_id(account_id),
             'currency': currency,
             'security_schema': 'Simple',
         }))
@@ -54,14 +59,14 @@ class Client(object):
         """
         logger.debug('Creating account with Wallet4 security schema: account_id: {}, currency: {} '
                      'containers: (user: {}, operator: {})'
-                     .format(account_id, currency, user_container, operator_container))
+                     .format(self.get_prefixed_account_id(account_id), currency, user_container, operator_container))
 
         containers = {
             "USER": user_container.as_request(),
             "OPERATOR": operator_container.as_request(),
         }
         response = self.make_request('POST', '/accounts', json.dumps({
-            'id': str(account_id),
+            'id': self.get_prefixed_account_id(account_id),
             'currency': currency,
             'security_schema': 'Wallet4',
             'containers': containers,
@@ -72,13 +77,13 @@ class Client(object):
     def create_currency_issuer_account(self, account_id, signatures_required, containers, currency):
         logger.debug('Creating account with CurrencyIssuer security schema: account_id: {}, currency: {} '
                      'containers: {} signatures_required: {}'
-                     .format(account_id, currency, containers, signatures_required))
+                     .format(self.get_prefixed_account_id(account_id), currency, containers, signatures_required))
         if any(not isinstance(container, Container) for container in containers):
             raise ValueError("containers should be a list of Container instances")
 
         containers = [dict(role=str(i), **container.as_request()) for i, container in enumerate(containers)]
         response = self.make_request('POST', '/accounts', json.dumps({
-            'id': str(account_id),
+            'id': self.get_prefixed_account_id(account_id),
             'currency': currency,
             'security_schema': 'CurrencyIssuer',
             'signatures_required': signatures_required,
@@ -93,7 +98,7 @@ class Client(object):
             "OPERATOR": operator_container.as_request(),
         }
         return Account.from_server_response(self.make_request('PUT', '/accounts', json.dumps({
-            'id': account_id,
+            'id': self.get_prefixed_account_id(account_id),
             'currency': currency,
             'security_schema': 'Wallet4',
             'containers': containers,
@@ -104,34 +109,39 @@ class Client(object):
         balance = self.make_request(
             'GET',
             '/balances/{currency}/{external_id}'
-            .format(currency=currency, external_id=account_id)
+            .format(currency=currency, external_id=self.get_prefixed_account_id(account_id))
         )['balance']
         if balance.startswith('0x'):
             if balance == '0x':
-                return 0.
-            return float(int(balance, 16))
+                return 0
+            return int(balance, 16)
         return float(balance)
 
     def get_account(self, account_id, currency='BTC'):
-        return Account.from_server_response(
-            self.make_request('GET', '/accounts/{external_id}/{currency}'
-                              .format(external_id=account_id, currency=currency))
+        account_data = self.make_request('GET', '/accounts/{external_id}/{currency}'
+                                         .format(external_id=self.get_prefixed_account_id(account_id),
+                                                 currency=currency))
+        account_data['account']['externalId'] = self.remove_prefix_from_account_id(
+            account_data['account']['externalId']
         )
+        return Account.from_server_response(account_data)
 
     def build_transaction(self, account_id, currency, destination, amount):
         body = {
             "destination": destination,
             "amount": amount,
         }
-        return Wallet4Transaction(**self.make_request('POST', '/transactions/build/{external_id}/{currency}'
-                                                              .format(external_id=account_id, currency=currency),
+        return Wallet4Transaction(**self.make_request('POST',
+                                                      '/transactions/build/{external_id}/{currency}'
+                                                      .format(external_id=self.get_prefixed_account_id(account_id),
+                                                              currency=currency),
                                                       body=json.dumps(body)))
 
     def submit(self, account_id, transaction, currency):
         if not isinstance(transaction, BaseTransaction):
             raise ValueError('transaction should be instance of BaseTransaction class')
         return self.make_request('POST', '/transactions/submit/{external_id}/{currency}'
-                                 .format(external_id=account_id, currency=currency),
+                                 .format(external_id=self.get_prefixed_account_id(account_id), currency=currency),
                                  body=transaction.to_json())
 
     def sign_wallet4_transaction(self, transaction, account_id, currency):
@@ -146,7 +156,7 @@ class Client(object):
         response = self.make_request(
             'POST',
             '/transactions/build_recovery/{external_id}/{currency}/{address}'
-            .format(external_id=account_id, currency=currency, address=old_address)
+            .format(external_id=self.get_prefixed_account_id(account_id), currency=currency, address=old_address)
         )
         logger.debug('response: {}'.format(response))
         return RecoveryTransaction(response['operator'], response['recovery_transaction'], response['account_id'])
@@ -163,33 +173,26 @@ class Client(object):
             "operation_type": "grant"
         }
         transaction = self.make_request('POST',
-                                        '/transactions/build/{id}/{currency}'.format(id=account_id, currency=currency),
+                                        '/transactions/build/{id}/{currency}'
+                                        .format(id=self.get_prefixed_account_id(account_id), currency=currency),
                                         json.dumps(body))
         return GrantTransaction(**transaction)
 
     def make_request(self, method, uri, body=None):
         url = self.ambisafe_server + uri
-        utctime = datetime.utcnow()
-        nonce = int(calendar.timegm(utctime.timetuple()) * 10 ** 3
-                    + utctime.microsecond * 10 ** -3)
-        message = "{}\n{}\n{}\n".format(nonce, method, url)
-        if body:
-            message += body
-        digest = hmac.new(self.api_secret, msg=message, digestmod=sha512)
-        signature = b64encode(digest.digest()).replace('\n', '')
-        headers = {
-            'API-key': self.api_key,
-            'signature': signature,
-            'timestamp': nonce,
-            'Accept': 'application/json'
-        }
+        headers = {'Accept': 'application/json'}
         if method in ['POST', 'PUT']:
             headers['Content-Type'] = 'application/json'
-        logger.debug('Request to ambisafe KeyServer: method: {}, url: "{}", headers: {}, data: {}'
+        logger.debug(u'Request to ambisafe KeyServer: method: {}, url: "{}", headers: {}, data: {}'
                      .format(method, url, headers, body))
-        response = requests.request(method, url, headers=headers, data=body)
-        logger.debug('Response from ambisafe KeyServer: status: {}, text: {}'
+        response = requests.request(method, url, headers=headers, data=body,
+                                    auth=AmbisafeAuth(self.api_key, self.api_secret))
+        logger.debug(u'Response from ambisafe KeyServer: status: {}, text: {}'
                      .format(response.status_code, response.text))
+        return Client._handle_response(response)
+
+    @staticmethod
+    def _handle_response(response):
         try:
             response_data = response.json()
         except ValueError, e:
