@@ -61,6 +61,23 @@ class ClientTestCase(unittest.TestCase):
     def setUpClass(cls):
         cls.client = Client('http://localhost:8080', 'test', 'demo', 'demo')
 
+    def test_wrong_client_url(self):
+        self.assertRaises(ValueError, Client, 'wrong_url', 'test', 'demo', 'demo')
+
+    def test_remove_prefix_from_account_id(self):
+        client = Client('http://localhost:8080', 'test', 'demo', 'demo', 'prefix')
+        self.assertEqual(client.remove_prefix_from_account_id('prefix:1'), '1')
+
+        client = Client('http://localhost:8080', 'test', 'demo', 'demo')
+        self.assertEqual(client.remove_prefix_from_account_id('prefix:1'), 'prefix:1')
+
+    def test_get_prefixed_account_id(self):
+        client = Client('http://localhost:8080', 'test', 'demo', 'demo', 'prefix')
+        self.assertEqual(client.get_prefixed_account_id('1'), 'prefix:1')
+
+        client = Client('http://localhost:8080', 'test', 'demo', 'demo')
+        self.assertEqual(client.get_prefixed_account_id('1'), '1')
+
     @httpretty.activate
     def test_create_account_wallet4(self):
         httpretty.register_uri(httpretty.POST, 'http://localhost:8080/accounts',
@@ -101,6 +118,25 @@ class ClientTestCase(unittest.TestCase):
         balance = self.client.get_balance(self.id, 'BTC')
         self.assertEqual(0.0011086, balance)
 
+    @httpretty.activate
+    def test_get_hex_balance(self):
+        url = 'http://localhost:8080/balances/ETH/{external_id}'.format(external_id=self.id)
+        httpretty.register_uri(httpretty.GET, url,
+                               body=json.dumps({u'address': u'38gP2E6Fj2s3sx63urYPpSrgTwupHGvKsK',
+                                                u'balance': u'0x',
+                                                u'balanceInSatoshis': None,
+                                                u'currencySymbol': u'ETH'}))
+        balance = self.client.get_balance(self.id, 'ETH')
+        self.assertEqual(balance, 0)
+
+        httpretty.register_uri(httpretty.GET, url,
+                               body=json.dumps({u'address': u'38gP2E6Fj2s3sx63urYPpSrgTwupHGvKsK',
+                                                u'balance': u'0xF',
+                                                u'balanceInSatoshis': None,
+                                                u'currencySymbol': u'ETH'}))
+        balance = self.client.get_balance(self.id, 'ETH')
+        self.assertEqual(balance, 15)
+
     @httpretty.activate
     def test_get_account(self):
         url = 'http://localhost:8080/accounts/{external_id}/BTC'.format(external_id=self.id)
